"""
   Copyright (C) 2011-2015 Jiri Pirko <jiri@resnulli.us>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
"""

__author__ = """
jiri@resnulli.us (Jiri Pirko)
"""

import capi
import select
import struct

TEAM_ANY_CHANGE = capi.TEAM_ANY_CHANGE
TEAM_PORT_CHANGE = capi.TEAM_PORT_CHANGE
TEAM_OPTION_CHANGE = capi.TEAM_OPTION_CHANGE

class TeamError(Exception):
    pass

class TeamLibError(TeamError):
    def __init__(self, msg, err = 0):
        self._msg = msg
        self._err = err

    def __str__(self):
        msg = self._msg
        if self._err:
            msg += " Err=%d" % self._err
        return msg

class TeamUnknownOptionTypeError(TeamError):
    pass

class TeamNetDeviceIndexNameConverter(object):
    """
    Class does conversion between various network device identificators.
    Note both functions have DeviceID as input parameter.
    That can be either: interface index (int)
                        interface name (str)
                        TeamNetDevice (or inheritor) instance
    """
    def __init__(self, th):
        self._th = th

    def get_ifindex(self, dev_id):
        """
        Get interface index.
        """
        if isinstance(dev_id, int):
            return dev_id
        elif isinstance(dev_id, TeamNetDevice):
            return dev_id.ifindex
        elif isinstance(dev_id, str):
            return capi.team_ifname2ifindex(self._th, dev_id)
        raise TeamError("Cannot convert to interface index.")

    def dev_ifname(self, dev_id):
        """
        Get interface name.
        """
        if isinstance(dev_id, str):
            return dev_id
        elif isinstance(dev_id, TeamNetDevice):
            ifindex = dev_id.ifindex
        elif isinstance(dev_id, int):
            ifindex = dev_id
        else:
            raise TeamError("Cannot convert to interface name.")
        return capi.team_ifindex2ifname(self._th, ifindex, 32)

class TeamNetDevice(object):
    """
    Class for manipulating generic network device.
    """
    def __init__(self, th, ifindex = 0):
        self._th = th
        self._conv = TeamNetDeviceIndexNameConverter(th)
        self.ifindex = ifindex

    def __str__(self):
        return self.ifname

    @property
    def ifindex(self):
        return self._ifindex

    @ifindex.setter
    def ifindex(self, ifindex):
        self._ifindex = ifindex
        self.ifname = self._conv.dev_ifname(ifindex)

    def get_hwaddr(self):
        err, hwaddr = capi.team_hwaddr_get(self._th, self.ifindex, 6)
        if err:
            raise TeamLibError("Failed to get hardware address", err)
        return ":".join(map(lambda x: "%02X" % x, struct.unpack('BBBBBB', hwaddr)))

    def set_hwaddr(self, hwaddr_str):
        pack = struct.pack('BBBBBB', *map(lambda x : int(x, 16), hwaddr_str.split(":")))
        err = capi.team_hwaddr_set(self._th, self.ifindex, pack)
        if err:
            raise TeamLibError("Failed to set hardware address", err)

class TeamPort(TeamNetDevice):
    """
    Class stores port data and serves for port modification.
    """
    def update(self, lib_port):
        """
        Update option by give library structure.
        """
        self.speed = capi.team_get_port_speed(lib_port)
        self.duplex = capi.team_get_port_duplex(lib_port)
        self.changed = capi.team_is_port_changed(lib_port)
        self.linkup = capi.team_is_port_link_up(lib_port)
        self.removed = capi.team_is_port_removed(lib_port)

class TeamPortListIterator(object):
    """
    Iterator class for TeamPortList class for iterating over all listed ports.
    """
    def __init__(self, ports):
        self._ports = ports
        self._cursor = 0

    def __iter__(self):
        iter

    def next(self):
        """ Get next item in dict """
        if self._cursor == len(self._ports):
            raise StopIteration
        else:
            key = self._ports.keys()[self._cursor]
            self._cursor += 1
            return self._ports[key]

class TeamPortList(object):
    """
    Class contains list of ports present on team. Dictionary is used
    internaly since port interface index is unique.
    """
    def __init__(self, th):
        self._th = th
        self._conv = TeamNetDeviceIndexNameConverter(th)
        self._ports = {}
        self.update()

    def __len__(self):
        return len(self._ports)

    def __iter__(self):
        return TeamPortListIterator(self._ports)

    def get_port(self, port_dev_id):
        """
        Get port instance identified by DeviceID.
        """
        return self._ports[self._conv.get_ifindex(port_dev_id)]

    def update(self):
        """
        Fetch fresh data from library and adjust update dictionary and port
        instances by it.
        """
        lib_port_ifindex_list = []
        lib_port = capi.team_get_next_port(self._th, None)
        while lib_port:
            ifindex = capi.team_get_port_ifindex(lib_port)
            lib_port_ifindex_list.append(ifindex)
            try:
                port = self.get_port(ifindex)
            except KeyError:
                port = TeamPort(self._th, ifindex)
                self._ports[ifindex] = port
            port.update(lib_port)
            lib_port = capi.team_get_next_port(self._th, lib_port)

        for ifindex in self._ports.keys():
            if not ifindex in lib_port_ifindex_list:
                del self._ports[ifindex]

class TeamOption(object):
    """
    Class stores option data and serves for value modification.
    """
    def __init__(self, th, name):
        self._th = th
        self.name = name

    def _get_option_value(self, lib_option):
        opt_type = capi.team_get_option_type(lib_option)
        if opt_type == capi.TEAM_OPTION_TYPE_U32:
            return capi.team_get_option_value_u32(lib_option)
        elif opt_type == capi.TEAM_OPTION_TYPE_STRING:
            return capi.team_get_option_value_string(lib_option)
        else:
            raise TeamUnknownOptionTypeError()

    def update(self, lib_option):
        """
        Update option by give library structure.
        """
        self.value = self._get_option_value(lib_option)
        self.changed = capi.team_is_option_changed(lib_option)

    def set_value(self, value):
        """
        Set option value.
        """
        if isinstance(value, int):
            err = capi.team_set_option_value_by_name_u32(self._th, self.name,
                                                         value)
        elif isinstance(value, str):
            err = capi.team_set_option_value_by_name_string(self._th,
                                                            self.name,
                                                            value)
        else:
            raise TeamUnknownOptionTypeError()
        if err:
            raise TeamLibError("Failed to set option", err)

class TeamOptionListIterator(object):
    """
    Iterator class for TeamOptionList class for iterating over all options.
    """
    def __init__(self, options):
        self._options = options
        self._cursor = 0

    def __iter__(self):
        iter

    def next(self):
        """ Get next item in dict """
        if self._cursor == len(self._options):
            raise StopIteration
        else:
            key = self._options.keys()[self._cursor]
            self._cursor += 1
            return self._options[key]

class TeamOptionList(object):
    """
    Class contains list of options present on team. Dictionary is used
    internaly since option name is unique.
    """
    def __init__(self, th):
        self._th = th
        self._options = {}
        self.update()

    def __len__(self):
        return len(self._options)

    def __iter__(self):
        return TeamOptionListIterator(self._options)

    def get_option(self, opt_name):
        """
        Get option instance identified by name.
        """
        return self._options[opt_name]

    def update(self):
        """
        Fetch fresh data from library and adjust update dictionary and option
        instances by it.
        """
        lib_option_name_list = []
        lib_option = capi.team_get_next_option(self._th, None)
        while lib_option:
            opt_name = capi.team_get_option_name(lib_option)
            lib_option_name_list.append(opt_name)
            try:
                option = self.get_option(opt_name)
            except KeyError:
                option = TeamOption(self._th, opt_name)
                self._options[opt_name] = option
            try:
                option.update(lib_option)
            except TeamUnknownOptionTypeError:
                del self._options[opt_name]
            lib_option = capi.team_get_next_option(self._th, lib_option)

        for opt_name in self._options.keys():
            if not opt_name in lib_option_name_list:
                del self._options[opt_name]

class TeamChangeHandler(object):
    def __init__(self, func, func_priv, type_mask):
        self._func = func
        self._func_priv = func_priv
        self._type_mask = type_mask

    def call(self, curr_type_mask):
        if self._type_mask & curr_type_mask:
            return self._func(self._func_priv)
        else:
            return 0

class TeamChangeHandlerList(object):
    def __init__(self):
        self._list = []

    def add(self, handler):
        if handler in self._list:
            raise TeamError("Failed to register change handler. Handler is already registered.")
        self._list.append(handler)

    def remove(self, handler):
        if not handler in self._list:
            raise TeamError("Failed to unregister change handler. Handler is not registered.")
        self._list.remove(handler)

    def call(self, type_mask):
        for handler in self._list:
            ret = handler.call(type_mask)
            if ret != 0:
                return ret


class Team(TeamNetDevice):
    """
    Class representing one team device instance.
    Paramaters passed to constructor allows:
        create == True ... Create new team device if it does not already exist.
        recreate == True ... Same as create but in case device exists already,
                             it's removed first.
        destroy == True ... Remove team device in close function.
    """
    def __init__(self, teamdev, create = False, recreate = False, destroy = False):
        th = capi.team_alloc()
        if not th:
            raise TeamLibError("Failed to allocate team handle.")

        super(Team, self).__init__(th)

        if isinstance(teamdev, str):
            err = 0
            if recreate:
                err = capi.team_recreate(th, teamdev)
            elif create:
                err = capi.team_create(th, teamdev)
            if err:
                raise TeamLibError("Failed to create team.", err)

        ifindex = self._conv.get_ifindex(teamdev) if teamdev else 0
        err = capi.team_init(th, ifindex)
        if err:
            raise TeamLibError("Failed to init team.", err)

        self.ifindex = ifindex
        self._destroy = destroy
        self._change_handler_list = TeamChangeHandlerList()
        self._port_list = TeamPortList(th)
        self._option_list = TeamOptionList(th)

        self._change_handler = capi.team_change_handler(self._change_handler_func,
                                                        TEAM_ANY_CHANGE)
        capi.py_team_change_handler_register(self._th, self._change_handler, None)


    def close(self):
        """
        Do class cleanup
        """
        if self._destroy:
            err = capi.team_destroy(self._th)
            if err:
                raise TeamLibError("Failed to destroy team.", err)

        capi.py_team_change_handler_unregister(self._th,
                                               self._change_handler, None)
        capi.team_free(self._th)

    def kill_loop(self):
        self._kill_loop = True

    def loop_forever(self):
        self._kill_loop = False
        fd = capi.team_get_event_fd(self._th)
        while True:
            try:
                ret = select.select([fd], [], [])
                if fd in ret[0]:
                    capi.team_handle_events(self._th)
            except KeyboardInterrupt:
                return
            except select.error as e:
                if e[0] == 4:
                    if self._kill_loop:
                        return
            except:
                raise

    def check_events(self):
        capi.team_check_events(self._th)

    def _change_handler_func(self, func_priv, type_mask):
        if type_mask & TEAM_PORT_CHANGE:
            self._port_list.update()
        if type_mask & TEAM_OPTION_CHANGE:
            self._option_list.update()
        self._change_handler_list.call(type_mask)

    def change_handler_register(self, change_handler):
        self._change_handler_list.add(change_handler)

    def change_handler_unregister(self, change_handler):
        self._change_handler_list.remove(change_handler)

    def get_mode_name(self):
        err, name = capi.team_get_mode_name(self._th)
        if err:
            raise TeamLibError("Failed to get mode name.", err)
        return name

    def set_mode_name(self, name):
        err = capi.team_set_mode_name(self._th, name)
        if err:
            raise TeamLibError("Failed to set mode name.", err)

    def get_active_port(self):
        err, port_ifindex = capi.team_get_active_port(self._th)
        if err:
            raise TeamLibError("Failed to get active port.", err)
        return self.get_port(port_ifindex)

    def set_active_port(self, dev_port_id):
        err = capi.team_set_active_port(self._th,
                                        self._conv.get_ifindex(dev_port_id))
        if err:
            raise TeamLibError("Failed to set active port.", err)

    def port_list(self):
        return self._port_list

    def get_port(self, port_dev_id):
        return self._port_list.get_port(port_dev_id)

    def port_add(self, port_dev_id):
        err = capi.team_port_add(self._th,
                                 self._conv.get_ifindex(port_dev_id))
        if err:
            raise TeamLibError("Failed to add port.", err)

    def port_remove(self, port_dev_id):
        err = capi.team_port_remove(self._th,
                                    self._conv.get_ifindex(port_dev_id))
        if err:
            raise TeamLibError("Failed to remove port.", err)

    def option_list(self):
        return self._option_list

    def get_option(self, opt_name):
        return self._option_list.get_option(opt_name)



#! /usr/bin/env python
"""
Basic test.

   Copyright (C) 2012-2015 Jiri Pirko <jiri@resnulli.us>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
"""

__author__ = """
jiri@resnulli.us (Jiri Pirko)
"""

import sys
import os
import getopt
import subprocess

def usage():
    """
    Print usage of this app
    """
    print "Usage: team_basic_test.py [OPTION...]"
    print ""
    print "  -h, --help                         print this message"
    print "  -c, --loop-count=NUMBER            number of loops (default 1)"
    print "  -p, --port=NETDEV                  port device (can be defined multiple times)"
    sys.exit()

class CmdExecFailedException(Exception):
    def __init__(self, retval):
        self.__retval = retval

    def __str__(self):
        return "Command execution failed: %s" % self.__retval

class CmdExecUnexpectedOutputException(Exception):
    def __init__(self, output, expected_output):
        self.__output = output
        self.__expected_output = expected_output

    def __str__(self):
        return "Command execution output unexpected: \"%s\" != \"%s\"" % (self.__output, self.__expected_output)

def print_output(out_type, string):
    print("%s:\n"
          "----------------------------\n"
          "%s"
          "----------------------------" % (out_type, string))

def cmd_exec(cmd, expected_output=None, cleaner=False):
    cmd = cmd.rstrip(" ")
    if not cleaner:
        print("# \"%s\"" % cmd)
    subp = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE)
    (data_stdout, data_stderr) = subp.communicate()

    if subp.returncode and not cleaner:
        if data_stdout:
            print_output("Stdout", data_stdout)
        if data_stderr:
            print_output("Stderr", data_stderr)
        raise CmdExecFailedException(subp.returncode)
    output = data_stdout.rstrip()
    if expected_output:
        if output != expected_output:
            raise CmdExecUnexpectedOutputException(output, expected_output)
    return output

class TeamBasicTest:
    def __init__(self):
        self._team_dev_name = "testteamx"
        self._team_modes = ["broadcast", "roundrobin", "activebackup", "loadbalance"]
        self._loop_count = 1
        self._port_names = []

    def set_loop_count(self, new_loop_count):
        self._loop_count = new_loop_count

    def port_add(self, port_name):
        self._port_names.append(port_name)

    def _run_one_mode(self, mode_name):
        team_name = self._team_dev_name
        cmd_exec("ip link add %s type team" % team_name)
        try:
            self._created_teams.append(team_name)
            cmd_exec("teamnl %s getoption mode" % team_name, "*NOMODE*")
            cmd_exec("teamnl %s setoption mode %s" % (team_name, mode_name))
            cmd_exec("teamnl %s getoption mode" % team_name, mode_name)

            for port_name in self._port_names:
                cmd_exec("ip link set %s down" % port_name)
                cmd_exec("ip link set %s master %s" % (port_name, team_name))

            cmd_exec("ip link set %s up" % team_name)
            cmd_exec("ip addr add 192.168.241.231/24 dev %s" % team_name)
            cmd_exec("ip link set %s down" % team_name)
            cmd_exec("ip addr flush dev %s" % team_name)

            cmd_exec("teamnl %s options" % team_name)
            cmd_exec("teamnl %s ports" % team_name)

            for port_name in self._port_names:
                cmd_exec("ip link set %s nomaster" % port_name)
        finally:
                cmd_exec("ip link del %s" % team_name)

    def _create_team_initscript(self):
        f = open("/etc/sysconfig/network-scripts/ifcfg-%s" % self._team_dev_name, "w")
        f.write(
"""
DEVICE="%s"
DEVICETYPE="Team"
ONBOOT="no"
BOOTPROTO=none
NETMASK=255.255.255.0
IPADDR=192.168.241.231
TEAM_CONFIG='{"runner": {"name": "activebackup"}}'
""" % self._team_dev_name)
        f.close()
        cmd_exec("ifup %s" % self._team_dev_name)
    
    def _create_port_initscript(self, port_name):
        f = open("/etc/sysconfig/network-scripts/ifcfg-%s" % port_name, "w")
        f.write(
"""
DEVICE="%s"
DEVICETYPE="TeamPort"
ONBOOT="yes"
TEAM_MASTER="%s"
TEAM_PORT_CONFIG='{"prio": 10}'
""" % (port_name, self._team_dev_name))
        f.close()
        cmd_exec("ifup %s" % port_name)

    def _run_teamd_initscripts(self):
        os.makedirs("/tmp/team_test/")
        try:
            ifcfg_files = " ".join(["ifcfg-%s" % port_name for port_name in self._port_names])
            cmd_exec("tar --ignore-failed-read -C /etc/sysconfig/network-scripts/ -cf /tmp/team_test/initscripts_bkp.tar %s" % ifcfg_files)
            self._create_team_initscript()
            for port_name in self._port_names:
                self._create_port_initscript(port_name)
            cmd_exec("ifdown %s" % self._team_dev_name)
        except:
            cmd_exec("ifdown %s" % self._team_dev_name, cleaner=True)
            cmd_exec("ip link del %s" % self._team_dev_name, cleaner=True)
            raise
        finally:
            cmd_exec("tar -C /etc/sysconfig/network-scripts/ -xf /tmp/team_test/initscripts_bkp.tar")
            os.unlink("/tmp/team_test/initscripts_bkp.tar")
            os.removedirs("/tmp/team_test/")

    def _run_one_loop(self, run_nr):
        print "RUN #%d" % (run_nr)
        self._created_teams = []
        try:
            for mode_name in self._team_modes:
                self._run_one_mode(mode_name)
            self._run_teamd_initscripts()
        finally:
            cmd_exec("modprobe -r team_mode_loadbalance team_mode_roundrobin team_mode_activebackup team_mode_broadcast team");

    def run(self):
        for i in xrange(self._loop_count):
            self._run_one_loop(i + 1)

def main():
    try:
        opts, args = getopt.getopt(
            sys.argv[1:],
            "hc:p:",
            ["help", "loop-count=", "port="]
        )
    except getopt.GetoptError, err:
        print str(err)
        usage()

    btest = TeamBasicTest()

    for opt, arg in opts:
        if opt in ("-h", "--help"):
            usage()
        elif opt in ("-c", "--loop-count"):
            btest.set_loop_count(int(arg))
        elif opt in ("-p", "--port"):
            btest.port_add(arg)

    btest.run()

if __name__ == "__main__":
    main()



#! /usr/bin/env python
"""
Team monitor

   Copyright (C) 2011-2015 Jiri Pirko <jiri@resnulli.us>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
"""

__author__ = """
jiri@resnulli.us (Jiri Pirko)
"""

import sys
import team

def port_change_handler_func(t):
    print "------------------\nport change\n\tport list:"
    for port in t.port_list():
        print ("\tifname %s, linkup %d, changed %d, speed %d, duplex %d, removed %d" %
                        (port.ifname, port.linkup, port.changed,
                         port.speed, port.duplex, port.removed))
    return 0

def option_change_handler_func(t):
    print "------------------\noption change\n\toption list:"
    for option in t.option_list():
        print ("\t%s = %s (changed %d)" % (option.name, option.value,
                                           option.changed))
    return 0

def main():
    appname = sys.argv[0]
    args = sys.argv[1:]

    if len(args) < 1:
        print "Usage: %s TEAMDEV" % appname
        sys.exit()

    ifname = args[0]

    t = team.Team(ifname)

    port_change_handler = team.TeamChangeHandler(
                                    port_change_handler_func, t,
                                    team.TEAM_PORT_CHANGE)
    t.change_handler_register(port_change_handler)
    option_change_handler = team.TeamChangeHandler(
                                    option_change_handler_func, t,
                                    team.TEAM_OPTION_CHANGE)
    t.change_handler_register(option_change_handler)

    t.loop_forever()

if __name__ == "__main__":
    main()



#! /usr/bin/env python
"""
Team daemon example. Implements very basic active backup functionality.
Note that this application spawns its own team device instead of connecting
to an existing one.

   Copyright (C) 2011-2015 Jiri Pirko <jiri@resnulli.us>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
"""

__author__ = """
jiri@resnulli.us (Jiri Pirko)
"""

import sys
import getopt
import team
import signal

def usage():
    """
    Print usage of this app
    """
    print "Usage: team_daemon.py [OPTION...]"
    print ""
    print "  -h, --help                         print this message"
    print "  -t, --team-name=NAME               name of team device"
    print "  -p, --port=NETDEV                  port device (can be defined multiple times)"
    print "  -m, --mode=MODENAME                name of team mode"
    sys.exit()

class TeamDaemon(object):
    def __init__(self):
        def_team_name = "teamtest0"
        self._team_name = def_team_name
        def_mode_name = "roundrobin"
        self._mode_name = def_mode_name
        self._port_names = []
        self._old_active_hwaddr = None

        self._parse_cmdline()

        if self._team_name == def_team_name:
            print "Using default team name \"%s\"" % def_team_name

        if self._mode_name == def_mode_name:
            print "Using default mode \"%s\"" % def_mode_name

        print "Team name is \"%s\"" % self._team_name
        self._t = team.Team(self._team_name, create=True, destroy=True)

        self._t.set_mode_name(self._mode_name)
        for port_name in self._port_names:
            self._t.port_add(port_name)

        if self._mode_name == "activebackup":
            self._port_change_handler_ab = team.TeamChangeHandler(
                                    self._port_change_handler_ab_func, None,
                                    team.TEAM_PORT_CHANGE)
            self._t.change_handler_register(self._port_change_handler_ab)

        self._port_change_handler = team.TeamChangeHandler(
                                    self._port_change_handler_func, None,
                                    team.TEAM_PORT_CHANGE)
        self._t.change_handler_register(self._port_change_handler)

        self._option_change_handler = team.TeamChangeHandler(
                                    self._option_change_handler_func, None,
                                    team.TEAM_OPTION_CHANGE)
        self._t.change_handler_register(self._option_change_handler)

    def _parse_cmdline(self):
        try:
            opts, args = getopt.getopt(
                sys.argv[1:],
                "ht:p:m:",
                ["help", "team-name=", "port=", "mode="]
            )
        except getopt.GetoptError, err:
            print str(err)
            usage()

        for opt, arg in opts:
            if opt in ("-h", "--help"):
                usage()
            elif opt in ("-t", "--team-name"):
                self._team_name = arg
            elif opt in ("-p", "--port"):
                self._port_names.append(arg)
            elif opt in ("-m", "--mode"):
                self._mode_name = arg

    def _port_change_handler_func(self, unused):
        print "------------------\nport change\n\tport list:"
        for port in self._t.port_list():
            print ("\tifname %s, linkup %d, changed %d, speed %d, duplex %d, removed %d" %
                        (port.ifname, port.linkup, port.changed,
                         port.speed, port.duplex, port.removed))
        return 0

    def _option_change_handler_func(self, unused):
        print "------------------\noption change\n\toption list:"
        for option in self._t.option_list():
            print ("\t%s = %s (changed %d)" % (option.name, option.value,
                                               option.changed))
        return 0

    def _change_active_port(self, old_active, new_active):
        if old_active:
            old_active.set_hwaddr(self._old_active_hwaddr)

        self._t.set_active_port(new_active)
        mac = self._t.get_hwaddr()
        self._old_active_hwaddr = new_active.get_hwaddr()
        new_active.set_hwaddr(mac)
        print "*** new active port \"%s\" selected" % new_active

    def _port_change_handler_ab_func(self, unused):
        try:
            active = self._t.get_active_port()
            if active:
                print "*** current active port is \"%s\"" % active
                if active.linkup:
                    return
                elif active.changed:
                    print "*** active port \"%s\" lost link" % active
        except KeyError:
            active = None
            print "*** no current active port set"

        best = None
        for port in self._t.port_list():
            if port.linkup:
                if (not best or
                    port.speed > best.speed or
                    (port.speed == best.speed and port.duplex > best.duplex)):
                    best = port
        if best:
            self._change_active_port(active, best)

    def close(self):
        self._t.close()

    def loop_forever(self):
        self._t.loop_forever()

    def kill_loop(self):
        self._t.kill_loop()

def main():
    team_daemon = TeamDaemon()

    def kill_loop(signum, frame):
        team_daemon.kill_loop()

    signal.signal(signal.SIGINT, kill_loop)
    signal.signal(signal.SIGHUP, kill_loop)
    signal.signal(signal.SIGTERM, kill_loop)

    team_daemon.loop_forever()
    team_daemon.close()

if __name__ == "__main__":
    main()
